<?php
/* Exif reader v 1.2
   By Richard James Kendall 
   Bugs to richard@richardjameskendall.com 
   Free to use, please acknowledge me 

   To use, just include this file (with require, include) and call

	$obj = new MG2Exif();
   $obj->getExifData(filename);

   An array called $exifData will be populated with the exif tags and folders from the image.
*/ 

class MG2Exif {

	// HOLDS THE RAW EXIF SECTION
	var $exifBlock;

	// LENGTH OF EXIF DATA BLOCK
	var $dataLength;

	// HOLDS THE FORMATTED EXIF DATA
	var $exifData;

	// holds the number format used in the EXIF data (1 == moto, 0 == intel)
	var $align;

	// holds the lengths and names of the data formats
	var $formatLength = array(0, 1, 1, 2, 4, 8, 1, 1, 2, 4, 8, 4, 8);
	var $formatType   = array('', 'BYTE', 'STRING', 'USHORT', 'ULONG', 'URATIONAL', 'SBYTE', 'UNDEFINED', 'SSHORT', 'SLONG', 'SRATIONAL', 'SINGLE', 'DOUBLE');

	// data for EXIF enumeations
	var $Orientation          = array('', 'Top left', 'Top right', 'Bottom right', 'Bottom left', 'Left top', 'Right top', 'Right bottom', 'Left bottom');
	var $ResUnit              = array('', 'inches', 'inches', 'cm', 'mm', 'um');
	var $YCbCrPos             = array('', 'Centre of Pixel Array', 'Datum Points');
	var $ExpProg              = array('', 'Manual', 'Program', 'Apeture Priority', 'Shutter Priority', 'Program Creative', 'Program Action', 'Portrait', 'Landscape');
	var $LightSource          = array('Unknown', 'Daylight', 'Fluorescent', 'Tungsten (incandescent)', 'Flash', 'Fine Weather', 'Cloudy Weather', 'Share', 'Daylight Fluorescent', 'Day White Fluorescent', 'Cool White Fluorescent', 'White Fluorescent', 'Standard Light A', 'Standard Light B', 'Standard Light C', 'D55', 'D65', 'D75', 'D50', 'ISO Studio Tungsten');
	var $MeterMode            = array('Unknown', 'Average', 'Centre Weighted', 'Spot', 'Multi-Spot', 'Pattern', 'Partial');
	var $RenderingProcess     = array('Normal Process', 'Custom Process');
	var $ExposureMode         = array('Auto', 'Manual', 'Auto Bracket');
	var $WhiteBalance         = array('Auto', 'Manual');
	var $SceneCaptureType     = array('Standard', 'Landscape', 'Portrait', 'Night Scene');
	var $GainControl          = array('None', 'Low Gain Up', 'High Gain Up', 'Low Gain Down', 'High Gain Down');
	var $Contrast             = array('Normal', 'Soft', 'Hard');
	var $Saturation           = array('Normal', 'Low Saturation', 'High Saturation');
	var $Sharpness            = array('Normal', 'Soft', 'Hard');
	var $SubjectDistanceRange = array('Unknown', 'Macro', 'Close View', 'Distant View');
	var $FocalPlaneResUnit    = array('', 'inches', 'inches', 'cm', 'mm', 'um');
	var $SensingMethod        = array('', 'Not Defined', 'One-chip Colour Area Sensor', 'Two-chip Colour Area Sensor', 'Three-chip Colour Area Sensor', 'Colour Sequential Area Sensor', 'Trilinear Sensor', 'Colour Sequential Linear Sensor');

	// converts a number from intel (little endian) to motorola (big endian format)
	function ii2mm($intel) {
		$mm  = '';
		for ($i = strlen($intel)-2; $i > -1; $i-=2) {
			$mm.= substr($intel, $i, 2);
		}
		return $mm;
	}

	// gets a number from the EXIF data and converts if to the correct representation
	function getNumber($data, $start, $length) {
		$a = bin2hex(substr($data, $start, $length));
		if (!$this->align) {
			$a = $this->ii2mm($a);
		}
		return (int)hexdec($a);
	}

	// gets a rational number (num, denom) from the EXIF data and produces a decimal
	function getRational($data, $tag, $type) {
		$a = bin2hex($data);
		if (!$this->align) $a = $this->ii2mm($a);			// Intel to Motorola
		if ($this->align === 1) {
			$n = hexdec(substr($a, 0, 8));
			$d = hexdec(substr($a, 8, 8));
		} else {
			$d = hexdec(substr($a, 0, 8));
			$n = hexdec(substr($a, 8, 8));
		}
		if ($type === 'S' && $n > 2147483647) {
			$n -= 4294967296;
		}
		if ($n == 0) {
			$data = '0';
		}
		elseif ($d != 0)	{
			$data = $n / $d;
		}
		else {
			$data = sprintf('%d/%d', $n, $d);
		}
		return $data;
	}

	// OPENS THE JPEG FILE AND ATTEMPTS TO FIND THE EXIF DATA
	// kh_mod 0.4.1, changed
	function getExifData($file) {

		// INIT START VALUES
		$result			   = false;
		$fp				   = false;
		$this->dataLength = 0;
		$this->exifBlock  = array();
		$this->exifData   = array();

		do {
			if (!is_file($file))              break;
			if (!$fp = fopen($file, 'rb'))    break;
			if (fread($fp, 2) !== "\xFF\xD8") break;

			while (!feof($fp)) {
				for ($i = 0;$i < 7;$i++) {
					$sectionMarker = ord(fgetc($fp));		// get one byte and convert it to number
					if ($sectionMarker !== 255) break;
				}
				if ($sectionMarker === 255)    break 2;	// 255 = legal padding, but not so many
				if ($sectionMarker === 225)	 break;		// 225 = Exif data section marker
			}

			// PROCESS EXIF SECTION
			$sectionLength = bin2hex(fread($fp, 2));
			$sectionLength = hexdec($sectionLength);
			if ($sectionLength < 8) break;

			$this->dataLength = $sectionLength - 2;
			$this->exifBlock  = fread($fp, $this->dataLength);
			$result = $this->extractEXIFData();
		}
		while(0);

		if ($fp) fclose($fp);

		return $result ? $this->exifData:false;
	}

	// reads the EXIF header and if it is intact it calls readEXIFDir to get the data
	function extractEXIFData() {

		if (substr($this->exifBlock, 0, 4) !== 'Exif') return false;
		if (substr($this->exifBlock, 6, 2) === 'II') {
			$this->align = 0;
		}
		elseif (substr($this->exifBlock, 6, 2) === 'MM') {
			$this->align = 1;
		}
		else return false;

		$a = $this->getNumber($this->exifBlock, 8, 2);
		if ($a != 0x2a) return false;

		$first_offset = $this->getNumber($this->exifBlock, 10, 4);
		if ($first_offset < 8 || $first_offset > 16) {
			return false;
		}
		$this->readEXIFDir(substr($this->exifBlock, 14));
		return true;
	}

	// reads the tags from and EXIF IFD and if correct deals with the data
	function readEXIFDir($data) {

		$data_in = '';
		$number_dir_entries = $this->getNumber($data, 0, 2);
		for ($i = 0;$i < $number_dir_entries;$i++) {
			$dir_entry  = substr($data, 2 + 12 * $i);
			$tag        = $this->getNumber($dir_entry, 0, 2);
			$format     = $this->getNumber($dir_entry, 2, 2);
			$components = $this->getNumber($dir_entry, 4, 4);
			if ($format > 12) {
				return false;
			}
			$byte_count = $components * $this->formatLength[$format];

			// GREATER THAN 4 BYTES THAT MEANS IT'S A POINTER TO A VALUE
			if ($byte_count > 4) {
				$offset_val = ($this->getNumber($dir_entry, 8, 4));
				if (($offset_val + $byte_count) > ($this->dataLength - 6)) {
					return false;
				}
				$data_in = substr($this->exifBlock, $offset_val + 6);
			}
			// '$byte_count' 4 BYTES OR LESS THAT MEANS IT'S A VALUE
			else {
				$data_in = substr($dir_entry, 8);
			}

			// '0x8769' => ExifOffset, '0x8825' => GPSInfo, MS add
			if ($tag === 0x8769 || $tag === 0x8825) {
				$tmp = ($this->getNumber($data_in, 0, 4)) - 8;
				$this->readEXIFDir(substr($data, $tmp));
			}
			else {
				$this->dealWithTag($tag, $format, $data_in, $byte_count);
			}
		}
	}

	// takes an EXIF tag id and returns the string name of that tag
	function tagId2name($id) {
		switch ($id) {
			case 0x0001: return 'GPSLatitudeRef';				// MS add
			case 0x0002: return 'GPSLatitude';					// MS add
			case 0x0003: return 'GPSLongitudeRef';				// MS add
			case 0x0004: return 'GPSLongitude';					// MS add
//			case 0x000e: return 'GPSTrackRef';					// kh_mod 0.4.5, add
//			case 0x000f: return 'GPSTrack';						// kh_mod 0.4.5, add
//			case 0x0012: return 'GPSMapDatum';					// kh_mod 0.4.5, add
//			case 0x001d: return 'GPSDateStamp';					// kh_mod 0.4.5, add
//			case 0x001e: return 'GPSDifferential';				// kh_mod 0.4.5, add
//			case 0x000b: return 'ACDComment';
//			case 0x00fe: return 'ImageType';
//			case 0x0106: return 'PhotometicInterpret';
//			case 0x010e: return 'ImageDescription';
			case 0x010f: return 'Make';
			case 0x0110: return 'Model';
			case 0x0112: return 'Orientation';
//			case 0x0115: return 'SamplesPerPixel';
			case 0x011a: return 'XRes';
//			case 0x011b: return 'YRes';
//			case 0x011c: return 'PlanarConfig';
//			case 0x0128: return 'ResUnit';
			case 0x0131: return 'Software';
			case 0x0132: return 'DateTime';
			case 0x013b: return 'Artist';
//			case 0x013e: return 'WhitePoint';					// kh_mod 0.2.0, changed
//			case 0x013f: return 'PrimaryChromaticities';		// kh_mod 0.4.5, add
//			case 0x0211: return 'YCbCrCoefficients';
//			case 0x0213: return 'YCbCrPos';
//			case 0x0214: return 'RefBlackWhite';
			case 0x8298: return 'Copyright';
			case 0x829a: return 'ExposureTime';
			case 0x829d: return 'FNumber';
//			case 0x8822: return 'ExpProg';
			case 0x8827: return 'ISOSpeedRating';
//			case 0x8830: return 'SensitivityType';				// kh_mod 0.4.5, add
//			case 0x8832: return 'RecommendedExposureIndex';	// kh_mod 0.4.5, add
			case 0x9003: return 'DTOpticalCapture';
//			case 0x9004: return 'DTDigitised';
//			case 0x9102: return 'CompressedBitsPerPixel';
//			case 0x9201: return 'ShutterSpeed';
//			case 0x9202: return 'ApertureWidth';
//			case 0x9203: return 'Brightness';
			case 0x9204: return 'ExposureBias';
//			case 0x9205: return 'MaxApertureWidth';			// kh_mod 0.1.0, changed
//			case 0x9206: return 'SubjectDistance';
//			case 0x9207: return 'MeterMode';
//			case 0x9208: return 'LightSource';
			case 0x9209: return 'Flash';
			case 0x920a: return 'FocalLength';
//			case 0x9213: return 'ImageHistory';
//			case 0x927c: return 'MakerNote';
//			case 0x9286: return 'UserComment';
//			case 0x9290: return 'SubsecTime';
//			case 0x9291: return 'SubsecTimeOrig';
//			case 0x9292: return 'SubsecTimeDigi';
//			case 0xa000: return 'FlashPixVersion';
			case 0xa001: return 'ColorSpace';
//			case 0xa002: return 'ImageWidth';
//			case 0xa003: return 'ImageHeight';
//			case 0xa005: return 'InteroperabilityOffset';	// kh_mod 0.4.5, add
//			case 0xa20e: return 'FocalPlaneXRes';
//			case 0xa20f: return 'FocalPlaneYRes';
//			case 0xa210: return 'FocalPlaneResUnit';
//			case 0xa217: return 'SensingMethod';
//			case 0xa300: return 'ImageSource';
//			case 0xa301: return 'SceneType';
//			case 0xa401: return 'RenderingProcess';
//			case 0xa402: return 'ExposureMode';
//			case 0xa403: return 'WhiteBalance';
//			case 0xa404: return 'DigitalZoomRatio';
//			case 0xa405: return 'FocalLength35mm';
//			case 0xa406: return 'SceneCaptureType';
//			case 0xa407: return 'GainControl';
//			case 0xa408: return 'Contrast';
//			case 0xa409: return 'Saturation';
//			case 0xa40a: return 'Sharpness';
//			case 0xa40c: return 'SubjectDistanceRange';
//			case 0xa430: return 'CameraOwner';					// kh_mod 0.4.5, add
//			case 0xa432: return 'LensSpecification';			// kh_mod 0.4.5, add
//			case 0xa434: return 'LensModel';						// kh_mod 0.4.5, add
//			case 0xa435: return 'LensSerialNumber';			// kh_mod 0.4.5, add
		}
		return false;
	}

	// takes a (U/S)(SHORT/LONG) checks if an enumeration for this value exists and if it does returns the enumerated value for $tvalue
	function enumValue($tag, $valueId) {

		$tname = $this->tagId2name($tag);
		return (!empty($tname) && isset($this->{$tname}[$valueId]))?
				 $this->{$tname}[$valueId]
				 :
				 $valueId;
	}

	// takes the USHORT of the flash value, splits it up into itc component bits and returns the string it represents
	// kh_mod 0.4.0 b4, changed
	function flashValue($bin) {
		$retval     = '';
		$bin        = sprintf('%08b', $bin);
		$flashfired = substr($bin, 7, 1);
		$returnd    = substr($bin, 5, 2);
		$flashmode  = substr($bin, 3, 2);
		$redeye     = substr($bin, 1, 1);
		if ($flashfired === '1') {
			$retval = 'Fired';
		} elseif ($flashfired === '0') {
			$retval = 'Did not fire';
		}
		if ($returnd === '10') {
			$retval .= ', Strobe return light not detected';
		} elseif ($returnd === '11') {
			$retval .= ', Strobe return light detected';
		}
		if ($flashmode === '01' || $flashmode === '10') {
			$retval .= ', Compulsory mode';
		} elseif ($flashmode === '11') {
			$retval .= ', Auto mode';
		}
		if ($redeye) {
			$retval .= ', Red eye reduction';
		} else {
			$retval .= ', No red eye reduction';
		}
		return $retval;
	}

	// takes a tag id along with the format, data and length of the data and deals with it appropriatly
	function dealWithTag($tag, $format, $data, $length) {
		if (!$tagName = $this->tagId2name($tag))	return;

		switch ($this->formatType[$format]) {
			case 'STRING':
				if (empty($this->exifData['ThumbData'])
					 && ($offset = strpos(strrev($data), "\xFF\xD8\xFF"))
					 && ($stop = strpos($data, "\xFF\xD9", strlen($data) - $offset)))
				{
					$this->exifData['ThumbData'] = substr($data, -$offset - 3, $stop);
				}
				$val = trim(substr($data, 0, $length));
				$this->setExifData($tagName, $val);
				break;
			case 'ULONG':
			case 'SLONG':
				$val = $this->enumValue($tag, $this->getNumber($data, 0, 4));
				$this->setExifData($tagName, $val);
				break;
			case 'USHORT':
			case 'SSHORT':
				switch ($tag) {
					case 0x9209:
						$val = $this->flashValue($this->getNumber($data, 0, 2));
						break;
					case 0xa001:
						$val = ($this->getNumber($data, 0, 2) === 1)?
								 'sRGB'
								 :
								 'Uncalibrated';
						break;
					default:
						$val = $this->enumValue($tag, $this->getNumber($data, 0, 2));
						break;
				}
				$this->setExifData($tagName, $val);
				break;
			case 'URATIONAL':
				// *** GPS, MS add *** //
				switch ($tag) {
					case 0x0002:
					case 0x0004:
						if (($tag === 0x0002) && ($this->exifData['GPSLatitudeRef']  === 'S')) {
							$fortegn = -1;
						} elseif (($tag === 0x0004) && ($this->exifData['GPSLongitudeRef'] === 'W')) {
							$fortegn = -1;
						} else {
							$fortegn = 1;
						}

						foreach(array(0, 8, 16) as $offset) {
							$nextval = $this->getRational(substr($data, $offset, 8), $tag, 'U');

							switch ($offset) {
								case  0: $deg_value = floor($nextval);
								         $min_value = ($nextval - $deg_value) * 60;
											break;
								case  8: $min_value+= floor($nextval);
								         $sec_value = ($nextval - $min_value) * 60;
											break;
								case 16: $sec_value+= $nextval;
							}
						}

						if ($tag === 0x0002) {
							$this->exifData['GPSDecLat'] = $fortegn * ($deg_value + $min_value/60 + $sec_value/3600);
						} else {
							$this->exifData['GPSDecLong'] = $fortegn * ($deg_value + $min_value/60 + $sec_value/3600); 
						}
						$val = $deg_value .'&deg; ' . $min_value .'&prime; '. round($sec_value,2) .'&Prime; ';
						break;
					// *** end GPS, MS add *** //
					default:
						$val = $this->getRational(substr($data, 0, 8), $tag, 'U');	// kh_mod 0.1.0, changed
						switch ($tag) {
							case 0x829a: // Exposure Time
								if (is_float($val)) {
									$val = ($val > .549)?
											 round($val, 1)
											 :
											 $this->convert2fraction($val);
								}
								break;
							case 0x920a: // Focal Length
								if (is_float($val)) {
									$val = round($val, 1);
								}
								break;
							case 0x9202: // Aperature Values (APEX Mode)
							case 0x9205:
								if (is_float($val)) {
									$val = exp(($val * log(2))/2);
									$val = round($val, 1);
								}
								break;
							case 0xa20e:
							case 0xa20f:
								if (is_float($val)) {
									$val = round($val, 2);
								}
						}
				}
				$this->setExifData($tagName, $val);
				break;
			case 'SRATIONAL':
				$val = $this->getRational(substr($data, 0, 8), $tag, 'S');			// kh_mod 0.1.0, changed
				if ($tag === 0x9201) { // Shutter Speed (APEX Mode)
					if (is_numeric($val)) {
						$val = 1/exp($val * log(2));
						$val = ($val > .549)?
								 round($val, 1)
								 :
								 $this->convert2fraction($val);
					}
					else {
						$val = '';
					}
				}
				elseif ($tag === 0x9204) { // ExposureBias
					if (is_float($val)) {
						$val = round($val, 2);
					}
				}
				$this->setExifData($tagName, $val);
				break;
			case 'UNDEFINED':
				switch ($tag) {
					case 0xa300:
						$val = ($this->getNumber($data, 0, 2) === 3)?
								 'Digital Camera'
								 :
								 'Unknown';
						$this->setExifData($tagName, $val);
						break;
					case 0xa301:
						$val = ($this->getNumber($data, 0, 2) === 3)?
								 'Directly Photographed'
								 :
								 'Unknown';
						$this->setExifData($tagName, $val);
						break;
				}
				break;
		}
	}

	// SET THE EXIF DATA INTO THE RESULT ARRAY
	function setExifData($tagName, $val) {
		$this->exifData[$tagName] = $val;
	}

	// CONVERT A FLOATING POINT NUMBER TO FRACTION
	// kh_mod 0.1.0, add
	function convert2fraction($float) {

		// IF FLOAT LESS THEN 1
		if ($float < 1)	{
			$MaxTerms	  = 15;				// limit to avoid infinite loop
			$MinDivisor	  = 0.000001;		// limit to avoid divide by zero
			$MaxPrecision = 0.00000001;	// break-off condition

			$n_un   = 1;						// initialize fractions
			$d_un   = 0;						// with 1/0, 0/1
			$n_deux = 0;
			$d_deux = 1;

			$f = $float;						// start value
			for ($i=0; $i<$MaxTerms; $i++) {
				$a = floor($f);				// get next term
				$f = $f - $a;					// get new divisor
				$n = $n_un * $a + $n_deux; // calculate new fraction
				$d = $d_un * $a + $d_deux;
				$n_deux = $n_un;				// save last two fractions
				$d_deux = $d_un;
				$n_un   = $n;
				$d_un   = $d;
				$fPrecs = abs($float - ($n/$d));

				if ($fPrecs < $MaxPrecision)	break;
				if ($f < $MinDivisor)			break;

				$f = 1/$f;
			}

			$data = (($d > 10) || ($d % $n === 0))?
					  '1/'. round($d/$n)
					  :
					  $n .'/'. $d;
		}
		// FLOAT IS GREATER OR EQUAL 1
		else {
			$data = round($float, 1);
		}

		return $data;
	}
}
?>