<?php
//
// KEYWORDS CLASS, IMPLEMENTATION SHOULD BE AS SINGLETON
// kh_mod 0.4.0 b3, add
class MG2Keywords {

	var $mg2;					// parent object reference
	var $keywordData;			// array: keyword data
	var $keywordIndex;		// array: item - keyword relations
	var $nameDBFlatData;		// name flat file: keyword data
	var $nameDBFlatIndex;	// name flat file: item - keyword relations
	var $pathDBFlatData;		// default flat file path: 'keyword data'
	var $pathDBFlatIndex;	// default flat file path: 'item - keyword relations'
	var $loadStatus;			// data and id read status (true/false)

	// INITIALIZE KEYWORDS
	function MG2Keywords(&$parent) {
		$this->mg2				  = &$parent;
		$this->nameDBFlatData  = 'mg2db_keywords';
		$this->nameDBFlatIndex = 'mg2db_item_keyword';
		$this->pathDBFlatData  = DATA_FOLDER . $this->nameDBFlatData  .'.php';
		$this->pathDBFlatIndex = DATA_FOLDER . $this->nameDBFlatIndex .'.php';
		$this->loadStatus		  = $this->loadKeywordDB();
	}

	// GET NUMBER OF ALL KEYWORDS (ADMIN ONLY)
	function getNumKeywords() {
		if ($this->loadStatus !== true) return false;

		return count($this->keywordData);
	}

	// GET NUMBER OF ALL ITEM - KEYWORD RECORDS (ADMIN ONLY)
	function getNumItemKeywords() {
		if ($this->loadStatus !== true) return false;

		$numItemKeywords = 0;
		foreach($this->keywordIndex as $strKeywordIDs) {
			$numItemKeywords += count(explode(',', $strKeywordIDs));
		}

		return $numItemKeywords;
	}

	// GET NUMBER OF ITEMS OF EACH KEYWORD (ADMIN ONLY)
	function getNumItemAllocation() {
		if ($this->loadStatus !== true) return false;

		$itemAllocation = array();
		foreach($this->keywordIndex as $itemID=>$strKeywordIDs) {
			foreach(explode(',', $strKeywordIDs) as $keywordID) {
				$itemAllocation[$keywordID][$itemID] = 1;
			}
		}

		return $itemAllocation;
	}

	// GET THE KEYWORD OF A KEYWORD ID (ADMIN ONLY)
	function getKeywordName($keywordID) {

		return (isset($this->keywordData[$keywordID]))?
				 $this->keywordData[$keywordID][2]
				 :
				 $this->getKeywordDefaults(2);
	}

	// GET ALL KEYWORDS OF AN ITEM (HTML)
	function getKeywords($itemID, $sorting=false) {

		return ($this->mg2->sqldatabase)?
				 $this->getKeywordsSQL($itemID, $sorting)
				 :
				 $this->getKeywordsFlat($itemID, $sorting);
	}

	function getKeywordsSQL($itemID, $sorting) {

		// INIT RETURN VALUE
		$keywords = array();

		// SET KEYWORD ORDER
		$order = $sorting ? 't2.keywordName':'t1.position';

		// GET KEYWORDS FROM DB
		$query  = sprintf(
						'SELECT t2.keywordID, t2.keywordName FROM `mg2db_item_keyword` t1, `mg2db_keywords` t2
						 WHERE t1.itemID = %d
						 %s
						 %s
						 AND   t1.keywordID = t2.keywordID
						 ORDER BY '. $order,
						 $itemID,
						 !defined('USER_ADMIN') ? 'AND t2.position  >= 0':'',
						 !defined('USER_ADMIN') ? 'AND t2.timestamp <= '. time():''
					 );
		$result = mg2mysql_query($query);

		while ($result && $record = mg2mysql_fetch_assoc($result)) {
			$keywordID				 = (int)$record['keywordID'];
			$keywords[$keywordID] = $record['keywordName'];
		}

		return $keywords;
	}

	function getKeywordsFlat($itemID, $sorting) {

		// INIT RETURN VALUE
		$keywords = array();

		do {
			if (!$this->loadStatus)										break;
			if (!isset($this->keywordIndex[$itemID]))				break;
			if (!$keywordIDs = $this->getKeywordIDs($itemID))	break;

			// IS SET ALPHABETICAL ORDER
			if ($sorting) {
				$this->mg2->sortIndexDB($keywordIDs, 'MG2Keywords->keywordData', 2);
			}

			foreach($keywordIDs as $keywordID) {
				if (!$this->checkKeywordID($keywordID)) continue;

				// DO SAVE ITEM'S KEYWORDS
				$keywords[$keywordID] = $this->getKeywordName($keywordID);
			}
		}
		while(0);

		return $keywords;
	}

	// GET ALL KEYWORD FOLDERS (ADMIN ONLY)
	function getAllKeywordFolders() {

		// INIT RETURN VALUE
		$folders = array();

		if ($this->loadStatus === true) {
			foreach(array_keys($this->keywordData) as $keywordID) {
				$folders[$keywordID] = $this->getKeywordFolder($keywordID);
			}
		}

		return $folders;
	}

	// WRAPPER TO ADD KEYWORDS TO ITEMS
	function addKeywordsToItems() {
		$this->applyKeywordsToItems('add', true);
	}

	// WRAPPER TO REMOVE KEYWORDS FROM ITEMS
	function remKeywordsToItems() {
		$this->applyKeywordsToItems('rem', false);
	}

	// APPLYS KEYWORDS TO SEVERAL ITEMS
	function applyKeywordsToItems($mode, $saveNewKewords) {

		$inputKeywordIDs  = $this->cleanKeywords($saveNewKewords);	// get IDs of new input keywords
		$selectedItemIDs  = $this->mg2->getSelectedItems();			// get IDs of selected items
		$numAffectedItems = 0;
		$error            = 0;

		do {
			if (count($inputKeywordIDs) === 0) { $error = 1; break; }
			if (count($selectedItemIDs) === 0) { $error = 2; break; }

			foreach ($selectedItemIDs as $itemID) {

				if ($this->updateItemKeywords($itemID, $inputKeywordIDs, $mode)) {
					$numAffectedItems++;
				}
			}

			// GET INPUT NAMES AS ARRAY
			$inputNames = array_map(array($this, 'getKeywordName'), $inputKeywordIDs);
		}
		while(0);

		$this->displayApplyResult (array (
					'number'   => $numAffectedItems,
					'keywords' => isset($inputNames) ? implode(', ', $inputNames):'',
					'error'    => $error,
					'mode'     => $mode
				 ));
	}

	// MESSAGE TO APPLY KEYWORDS
	function displayApplyResult($result) {

		// SUCCESS
		if ($result['error'] === 0) {
			$success = ($result['mode'] === 'add')? 'keywords_added':'keywords_removed';
			$this->mg2->setStatus(sprintf($this->mg2->lang[$success],
											$this->mg2->htmlchars2entities($result['keywords']),
											$result['number']
										 ));
		}
		// COULD NOT ADD RESP. REMOVE KEYWORDS
		elseif ($result['error'] === 1) {
			$failure = ($result['mode'] === 'add')? 'nokeywords_toadd':'nokeywords_torem';
			$this->mg2->setStatus($this->mg2->lang[$failure], 1);
		}
		// NO FILES WERE SELECTED
		elseif ($result['error'] === 2) {
			$this->mg2->setStatus($this->mg2->lang['filenotselected'], 1);
		}

		$this->mg2->displayStatus();
	}

	// SET KEYWORDS FOR ONE ITEM
	function setItemKeywords($itemID) {
		return $this->updateItemKeywords($itemID, $this->cleanKeywords());
	}

	/**
	 * Updates keywords for one item and returns the
	 * sum of changed, deleted and added keywords
	 *
	 * @param int $itemID
	 * @param array $inputIDs
	 * @param string $mode
	 *
	 * @return int | false
	 */
	function updateItemKeywords($itemID, $inputIDs, $mode='set') {
		if ($this->loadStatus !== true)	return false;

		// GET SAVED KEYWORD INDICES AS ARRAY
		$savedIDs = isset($this->keywordIndex[$itemID]) ?
						$this->getKeywordIDs($itemID)
						:
						array();

		// ADD INPUT TO ALREADY SAVED KEYWORDS
		if ($mode === 'add') {
			$changedIDs = array_diff($inputIDs, $savedIDs);								// keywords to add
			$inputIDs   = array_merge($savedIDs, $changedIDs);
		}
		// REMOVE INPUT TO ALREADY SAVED KEYWORDS
		elseif ($mode === 'rem') {
			$changedIDs = array();																// keywords to add
			$inputIDs   = array_diff($savedIDs, $inputIDs);								// left keywords
		}
		// SET INPUT AS NEW KEYWORDS
		elseif ($mode === 'set') {
			$intersect  = array_values(array_intersect($savedIDs, $inputIDs));	// not new keywords
			$changedIDs = array_diff_assoc($inputIDs, $intersect);					// to change, add
		}

		// GET DELETED IDs
		$deletedIDs = array_diff($savedIDs, $inputIDs);

		// CHANGED KEYWORD INPUT TO SAVED?
		if (count($changedIDs) || count($deletedIDs)) {

			// IF KEYWORD INPUT EMPTY, DELETE IT
			if (empty($inputIDs)) {
				unset($this->keywordIndex[$itemID]);
			}
			// SAVE KEYWORD INDICES AS STRING
			else {
				$this->keywordIndex[$itemID] = implode(',', $inputIDs);
			}

			// WRITE SQL "ITEM ID - KEYWORD ID" RELATION
			if ($this->mg2->sqldatabase) {
				$delOK = (!empty($inputIDs) && empty($changedIDs))?
							$this->writeKeywordDB('index', 'delexcept', array($itemID))
							:
							$this->writeKeywordDB('index', 'del', array($itemID));

				$setOK = (!empty($changedIDs) && ($delOK > -1))?
							$this->writeKeywordDB('index', 'new', array($itemID))
							:
							$delOK;
			}
			// WRITE FLAT FILE "mg2db_ikeyword.php"
			else {
				$setOK = $this->writeKeywordDB('index');
			}
		}
		// KEYWORDS ALREADY SET
		else {
			$setOK = 0;
		}

		// RETURNS SUM OF CHANGED, DELETED AND ADDED KEYWORDS
		return ($setOK > -1)?
				 count($changedIDs) + count($deletedIDs)
				 :
				 false;
	}

	// EDIT A NEW KEYWORD (ADMIN ONLY)
	function editNewKeyword() {

		$folderType		= 9;
		$valueID			= -1;
		$qualifierID	= 'keyID';
		$currentPage	= 1;
		$action			= 'buildNewKeyword';
		$dialog			= 'editKeyword';
		$headline		= $this->mg2->lang['newkeyword'];
		$icon				= $this->mg2->getFolderIcon();			// default icon
		$icon['url']	= '';
		$iconOptions	= false;
		$iconSorting	= false;
		$iconPwdLock	= false;
		$moveToFolder	= false;
		$folderName		= '';
		$introtext		= '';
		$publish			= $this->mg2->time2date($this->getKeywordDefaults(4));
		$position		= $this->getKeywordDefaults(5);
		$sortMode		= $this->getKeywordDefaults(7);
		$sortContentBy	= $sortMode & 15;								// 6 record index (name)
		$sortDirection	= $sortMode & 16;								// 0 means 'ascending'
		$fNumCols		= '';												// number of cols (folder setting)
		$fNumRows		= '';												// number of rows (folder setting)
		$localThumbSet	= false;											// grid checkbox (local setting)

		// DISPLAY FOLDER DIALOG
		$mg2 = &$this->mg2;
		include(ADMIN_FOLDER .'admin_folderdialog.php');
	}

	// SET NEW KEYWORD IN DATABASE
	function buildNewKeyword() {

		// INIT START VALUES
		$keywordID = $this->keywordData ? max(array_keys($this->keywordData)):0;
		$input	  = $this->cleanKeywordInput(++$keywordID);

		// IF KEYWORD NAME EMPTY
		if (!isset($input[2])) $input[2] = 'untitled_#'. $keywordID;

		// SAVE AND OPTIMIZE DATA RECORD
		$this->keywordData[$keywordID] = $input;
		if (!$this->mg2->sqldatabase) $this->optimizeKeywordData(array($keywordID));

		// WRITE DATABASE
		if ($this->writeKeywordDB('data', 'upd', array($keywordID)) > 0) {
			$this->mg2->log('Created new keyword \''. $input[2] .'\'');
			$message = sprintf($this->mg2->lang['keywordcreated'], $input[2]);
			$message.= $this->mg2->status;
			$this->mg2->displayStatus($message);
		}
		// WRITE ERROR
		else {
			$this->mg2->log('Couldn\'t create new \''. $input[2] .'\'');
			$this->mg2->displayStatus($this->mg2->lang['nosettingssaved'], 3);
		}

		return $keywordID;
	}

	// EDIT A KEYWORD (ADMIN ONLY)
	function editKeyword($keywordID) {

		// CHECK FOLDER ID
		if (!isset($this->keywordData[$keywordID])) {
			$this->mg2->displayStatus($this->mg2->lang['nokeywordid'] .' #'. $keywordID, 3);
			return false;	// invalid keyword id
		}

		$folderType		= 9;
		$valueID			= $keywordID;
		$qualifierID	= 'keyID';
		$currentPage	= 1;
		$action			= 'updateKeyword';
		$dialog			= 'editKeyword';
		$headline		= $this->mg2->lang['editkeyword'];
		$icon				= $this->mg2->getFolderIcon();			// default icon
		$icon['url']	= $this->mg2->getGalleryLink(array('search'=>$keywordID,'user'=>GALLERY_ID));
		$keywordRC		= $this->getKeywordFolder($keywordID);
		$iconOptions	= false;
		$iconSorting	= false;
		$iconPwdLock	= $keywordRC[8] ? true:false;
		$moveToFolder	= false;
		$folderName		= $this->mg2->htmlchars2entities($keywordRC[2]);
		$introtext		= $this->mg2->isJavaScriptEditor() ?
							  $keywordRC[3]								// WYSIWYG HTML-editor
							  :
							  $this->mg2->br2nl($keywordRC[3]);		// plain text-editor
		$introtext		= $this->mg2->htmlchars2entities($introtext);
		$publish			= $this->mg2->time2date($keywordRC[4]);
		$position		= $keywordRC[5];
		$sortMode		= $keywordRC[7];
		$sortContentBy	= $sortMode & 15;								// 6 record index (name)
		$sortDirection	= $sortMode & 16;								// 0 means 'ascending'
		$fNumCols		= '';												// number of cols (folder setting)
		$fNumRows		= '';												// number of rows (folder setting)
		$localThumbSet	= false;											// grid checkbox (local setting)

		// IF FOLDER SETTING ACTIVE FOR THUMB GRID
		if ($keywordRC[9] > 0 || $keywordRC[10] > 0) {
			$fNumCols = ($keywordRC[9]  > 0)? (int)$keywordRC[9] : $this->mg2->lang['default'];
			$fNumRows = ($keywordRC[10] > 0)? (int)$keywordRC[10]: $this->mg2->lang['default'];
			$localThumbSet = true;
		}

		// DISPLAY FOLDER DIALOG
		$mg2 = &$this->mg2;
		include(ADMIN_FOLDER .'admin_folderdialog.php');
	}

	// UPDATE KEYWORD (ADMIN ONLY)
	function updateKeyword($keywordID) {

		// CHECK FOLDER ID
		if (!isset($this->keywordData[$keywordID])) {
			return false;	// invalid keyword id
		}

		// INIT START VALUES
		$input	  = $this->cleanKeywordInput($keywordID);
		$keywordRC = $this->getKeywordFolder($keywordID);
		$updated	  = 0;

		// IF SORT BY EXIF DATE THAN SET EXIF DATE
		if ((($keywordRC[7] & 15) !== 13) && (($input[7] & 15) === 13)) {
			$itemIDs = $this->selectItemIDs($keywordID);
			$this->mg2->setExifDate($itemIDs); 
		}

		// SET KEYWORD CHANGES
		foreach($input as $key=>$value) {
			if ($keywordRC[$key] === $value) continue;

			$this->keywordData[$keywordID][$key] = $value;
			$updated++;
		}

		// DELETE PASSWORD
		if (!empty($_POST['delPwd']) && isset($this->keywordData[$keywordID][8])) {
			unset($this->keywordData[$keywordID][8]);
			$updated++;
		}

		// NOTHING TO DO
		if ($updated === 0) {
			$this->mg2->displayStatus($this->mg2->lang['nochanges2save'], 1);
		}
		// SAVE SETTING (WRITE DB)
		else {
			if (!$this->mg2->sqldatabase) $this->optimizeKeywordData(array($keywordID));
			// WRITE OK
			if ($this->writeKeywordDB('data', 'upd', array($keywordID)) > 0) {
				$this->mg2->log('Update of keyword folder \''. $keywordRC[2] .'\' complete.');
				$this->mg2->displayStatus($this->mg2->lang['keywordupdated']);
			}
			// WRITE ERROR
			else {
				$this->mg2->log('Couldn\'t update keyword folder \''. $keywordRC[2] .'\'!');
				$this->mg2->displayStatus($this->mg2->lang['nosettingssaved'], 3);
			}
		}
	}

	// CLEAN KEYWORD FORM (ADMIN ONLY)
	function cleanKeywordInput($keywordID) {

		// INIT RETURN VALUE
		$input = array();

		if (!empty($_POST['fname'])) {
			$newKeywordName = $this->mg2->charfix($_POST['fname']);
			$newKeywordName = strip_tags($newKeywordName);
			$newKeywordName = (string)substr($newKeywordName, 0, 255);
			$oldKeywordName = (string)$this->getKeywordName($keywordID);
			if ($this->checkNewKeywordName($newKeywordName, $oldKeywordName)) {
				$input[2] = $newKeywordName;
			}
		}
		if (isset($_POST['introtext'])) {
			$input[3] = $this->mg2->getFormTextarea('introtext');
		}
		if (isset($_POST['publish']))		$input[4]  = ($date = $this->mg2->date2time($_POST['publish']))?
																	 $date
																	 :
																	 $this->getKeywordDefaults(4);
		if (!empty($_POST['position']))	$input[5]  = (int)$_POST['position'];
		if (!empty($_POST['sortby']))		$input[7]  = (int)$_POST['sortby']    & 15;
		if (!empty($_POST['direction']))	$input[7] |= (int)$_POST['direction'] ? 16:0;
		if (!empty($_POST['password']))	$input[8]  = md5(strrev(md5(substr(trim($_POST['password']),0,MAXLEN_PWD))));
		if (isset($_POST['localCols']))  $input[9]  = (int)$_POST['localCols'];
		if (isset($_POST['localRows']))  $input[10] = (int)$_POST['localRows'];

		// CHECK PASSWORD CONFIRM, NEW KEYWORDS ONLY
		if (isset($_POST['confirm'])	&&
			 isset($input[8])				&& 
			 trim($_POST['password']) !== trim($_POST['confirm']))
		{
			unset($input[8]);
			$this->mg2->status = sprintf('<div style="margin-top:8px;color:#C69023">%s %s</div>',
											$this->mg2->lang['warning'],
											$this->mg2->lang['nopwdmatch']
										);
		}

		// SET COLS AND ROWS: USE GLOBAL DATA
		if (isset($_POST['thumbGrid']) && (int)$_POST['thumbGrid'] !== 1) {
			$input[9] = $input[10] = 0;
		}

		return $input;
	}

	// VERIFY NEW KEYWORD NAME
	function checkNewKeywordName($newKeywordName, $oldKeywordName) {

		$isKeyName = false;
		do {
			// KEYWORD EMPTY OR NOT CHANGED
			if ($this->mg2->string_empty($newKeywordName))	break;
			if ($newKeywordName === $oldKeywordName)			break;

			// CONVERT TO LOWER
			$keyToLower		  = $this->stringToLowerLambda();
			$oldKeywordLower = $keyToLower($oldKeywordName);
			$newKeywordLower = $keyToLower($newKeywordName);

			// DOES NEW KEYWORD ALREADY EXIST
			if ($oldKeywordLower !== $newKeywordLower)
			foreach(array_keys($this->keywordData) as $keywordID) {
				if ($keyToLower($this->keywordData[$keywordID][2]) === $newKeywordLower) {
					$this->mg2->displayStatus(sprintf(
															$this->mg2->lang['duplicatekeyword'],
															$this->keywordData[$keywordID][2]
													  ), 2);
					break 2;
				}
			}

			$isKeyName = true;
		}
		while(0);

		return $isKeyName;
	}

	// DELETE ONE KEYWORD (ADMIN ONLY)
	function deleteKeyword($keywordID) {
		if(!defined('USER_ADMIN')) return false;

		$keyName  = $this->getKeywordName($keywordID);
		$deleteOK = ($this->mg2->sqldatabase)?
						$this->deleteKeywordSQL($keywordID)
						:
						$this->deleteKeywordFlat($keywordID);

		// LOG AND DISPLAY MESSAGE
		if ($deleteOK !== false) {
			$this->mg2->log('Deleted keyword \''. $keyName .'\'');
			$this->mg2->displayStatus(sprintf($this->mg2->lang['keyworddeleted'], $keyName));
		}
		else {
			$this->mg2->log('Couldn\'t delete keyword \''. $keyName .'\'');
			$this->mg2->displayStatus($this->mg2->lang['keywordnotdeleted'], 3);
		}
	}

	// DELETE A KEYWORD SQL
	function deleteKeywordSQL($keywordID) {

		// DELETE RECORDS
		$query = 'DELETE FROM t1, t2
					 USING `mg2db_keywords` t1 LEFT JOIN `mg2db_item_keyword` t2
					 ON t1.keywordID = t2.keywordID
					 WHERE t1.keywordID = '. (int)$keywordID;
		$delOK = mg2mysql_query($query);

		// UPDATE KEYWORD DATA
		if (($numDelIDs = mg2mysql_affected_rows()) > 0) $this->loadKeywordDB();

		return $numDelIDs ? $numDelIDs:false;
	}

	// DELETE A KEYWORD FLATFILE
	function deleteKeywordFlat($keywordID) {

		// INIT VALUES
		$delIndexOK	= 0;
		$delDataOK	= 0;
		$numDelIDs	= 0;

		// DELETE KEYWORD IN "ITEM - KEYWORD" RELATION
		foreach(array_keys($this->keywordIndex) as $itemID) {
			if (!$keywordIDs = $this->getKeywordIDs($itemID))						 continue;	// no keywords
			if (($position = array_search($keywordID, $keywordIDs)) === false) continue;	// keyword doesn't exit

			// UPDATE ITEM ENTRY
			if (count($keywordIDs) > 1) {
				unset($keywordIDs[$position]);
				$this->keywordIndex[$itemID] = implode(',', $keywordIDs);
			}
			// DELETE ITEM ENTRY
			else {
				unset($this->keywordIndex[$itemID]);
			}

			$numDelIDs++;
		}
		if ($numDelIDs > 0) $delIndexOK = $this->writeKeywordDB('index');

		// DELETE KEYWORD ITSELF
		if (isset($this->keywordData[$keywordID])) {
			unset($this->keywordData[$keywordID]);
			$delDataOK = $this->writeKeywordDB('data');
		}

		return ($delIndexOK >= 0 && $delDataOK >= 0)? $numDelIDs:false;
	}

	// DELETE UNUSED KEYWORD RECORDS (ADMIN ONLY)
	function deleteUnusedKeywords() {
		if(!defined('USER_ADMIN')) return false;

		$numDeleted = ($this->mg2->sqldatabase)?
						  $this->deleteUnusedKeywordsSQL()
						  :
						  $this->deleteUnusedKeywordsFlat();

		// LOG AND DISPLAY MESSAGE
		if ($numDeleted === false) {
			$this->mg2->log('Couldn\'t delete all unused keywords');
			$this->mg2->displayStatus($this->mg2->lang['keywordsnotdeleted'], 3);
		}
		elseif ($numDeleted === 0) {
			$this->mg2->log('There aren\'t any unused keywords to delete');
			$this->mg2->displayStatus($numDeleted .' '. $this->mg2->lang['keywordsdeleted'], 1);
		}
		else {
			$this->mg2->log('Deleted unused keywords');
			$this->mg2->displayStatus($numDeleted .' '. $this->mg2->lang['keywordsdeleted']);
		}
	}

	// DELETE UNUSED KEYWORD RECORDS - SQL
	function deleteUnusedKeywordsSQL() {

		$query = 'DELETE FROM t1
					 USING `mg2db_keywords` t1 LEFT JOIN `mg2db_item_keyword` t2
					 ON t1.keywordID = t2.keywordID
					 WHERE t2.keywordID IS NULL';
		$delOK = mg2mysql_query($query);

		// UPDATE KEYWORD DATA
		if (($numDelIDs = mg2mysql_affected_rows()) > 0) $this->loadKeywordDB();

		return $delOK ? $numDelIDs:false;
	}

	// DELETE UNUSED KEYWORD RECORDS - FLATFILE
	function deleteUnusedKeywordsFlat() {

		$itemAllocation = $this->getNumItemAllocation();
		$numDeletedIDs  = 0;
		foreach (array_keys($this->keywordData) as $keywordID) {
			if (isset($itemAllocation[$keywordID])) continue;

			unset($this->keywordData[$keywordID]);
			$numDeletedIDs++;
		}

		// UPDATE KEYWORD DATA
		$delOK = ($numDeletedIDs > 0)? $this->writeKeywordDB('data'):0;

		return $delOK >= 0 ? $numDeletedIDs:false;
	}

	// DELETE ALL KEYWORDS
	function deleteAllKeywords() {

		// GET CURRENT NUMBER OF KEYWORDS
		$numKeywords = $this->getNumKeywords();

		// SAVE ALL KEYWORDS
		if ($numKeywords) {
			require_once(INC_FOLDER .'mg2admin_backup.php');
			$Backup = &$this->mg2->getInstance('MG2Backup');
			$Backup->backup_DB(array('keywords'));
		}

		$this->keywordIndex = array();
		$this->keywordData  = array();
		$delIndexOK = $this->writeKeywordDB('index', 'all');
		$delDataOK  = $this->writeKeywordDB('data', 'all');

		// LOG AND DISPLAY MESSAGE
		if ($numKeywords === 0) {
			$this->mg2->displayStatus($numKeywords .' '. $this->mg2->lang['keywordsdeleted'], 1);
		}
		elseif ($delIndexOK === 0 && $delDataOK === 0) {
			$this->mg2->log('Deleted all keywords');
			$this->mg2->displayStatus($this->mg2->lang['keywordsdeleted']);
		}
		else {
			$this->loadKeywordDB();
			$this->mg2->log('Couldn\'t delete all keywords');
			$this->mg2->displayStatus($this->mg2->lang['keywordsnotdeleted'], 3);
		}

		return array('data'=>$delDataOK, 'index'=>$delIndexOK);
	}

	// DELETE RECORDS WITHOUT RELATIONS TO 'all_items' (ADMIN ONLY)
	function deleteResidueIndex() {
		if(!defined('USER_ADMIN')) return false;

		return ($this->mg2->sqldatabase)?
				 $this->deleteResidueIndexSQL()
				 :
				 $this->deleteResidueIndexFlat();
	}

	// DELETE RECORDS WITHOUT RELATIONS TO 'mg2db_items' - SQL
	function deleteResidueIndexSQL() {

		$query = 'DELETE FROM t1
					 USING `mg2db_item_keyword` t1
					 LEFT JOIN `mg2db_items` t2 ON t1.itemID = t2.itemID
					 LEFT JOIN `mg2db_keywords` t3 ON t1.keywordID = t3.keywordID
					 WHERE t2.itemID IS NULL
					 OR    t3.keywordID IS NULL';

		$delOK = mg2mysql_query($query);

		// SET CHRONICLE DATA DIRTY
		if (($numDelIDs = mg2mysql_affected_rows()) > 0) $this->loadStatus = false;

		return $delOK ? $numDelIDs:false;
	}

	// DELETE RECORDS WITHOUT RELATIONS TO 'all_items' - FLATFILE
	function deleteResidueIndexFlat() {

		if (!$this->loadStatus) return;

		// INIT RETURN VALUE
		$nToDelete = 0;

		// CHECK EACH KEYWORD - ITEM RELATION
		$keywordDataIDs = array_keys($this->keywordData);
		foreach (array_keys($this->keywordIndex) as $itemID) {

			$keywordItemIDs = explode(',', $this->keywordIndex[$itemID]);
			$verifiedIDs	 = isset($this->mg2->all_items[$itemID]) ?
									array_intersect(
										$keywordItemIDs,
										$keywordDataIDs
									)
									:
									array();

			// NOTHING TO DO, NO CHANGES
			$delKeywordIDs = count($keywordItemIDs) - count($verifiedIDs);
			if ($delKeywordIDs === 0)						continue;

			// COUNT TO DELETE KEYWORD ENTRIES
			$nToDelete += $delKeywordIDs;

			// DELETE ALL KEYWORD ENTRIES FOR THIS ITEM
			if (count($verifiedIDs) === 0) {
				unset($this->keywordIndex[$itemID]);	continue;
			}

			// SAVE VERIFIED ITEM - KEYWORD ENTRIES
			$this->keywordIndex[$itemID] = implode(',', $verifiedIDs);;			
		}

		// WRITE CHANGES INTO 'mg2db_item_keyword.php'
		$writingOK = $nToDelete ?
						 $this->writeKeywordFlatfile('index')
						 :
						 0;

		return ($writingOK !== -1) ? $nToDelete:false;
	}

	// SELECT ALL ITEMS OF A KEYWORD
	function selectItemIDs($keywordID) {

		// INIT RETURN VALUE
		$itemIDs = false;

		do {
			// CHECK THIS KEYWORD ID
			if (!$this->checkKeywordID($keywordID)) {
				break;
			}

			// GET ALL VALID SUBFOLDER IDs
			if (!$validFolderIDs = $this->mg2->getValidSubfolders('all', 'unstruct')) {
				break;
			}

			// REMOVE PASSWORD PROTECTED FOLDERS
			if (!defined('USER_ADMIN')								&&
				 (empty($_SESSION[GALLERY_ID]['adminmode'])	||
				 $_SESSION[GALLERY_ID]['adminmode'] !== $this->mg2->adminpwd))
			{
				foreach (array_keys($validFolderIDs) as $folderID)
				{
					if (($folderPwd = trim($this->mg2->all_folders[$folderID][8]))	&&
						 (!isset($_SESSION[GALLERY_ID]['fPwdHash'][$folderID])		||
						 ($folderPwd !== $_SESSION[GALLERY_ID]['fPwdHash'][$folderID])))
					{
						unset($validFolderIDs[$folderID]);
						if (!$validFolderIDs)
						{
							break;
						}
					}
				}
			}

			// START ITEM SELECTION
			$itemIDs = ($this->mg2->sqldatabase)?
						  $this->selectItemIDsSQL($keywordID, $validFolderIDs)
						  :
						  $this->selectItemIDsFlat($keywordID, $validFolderIDs);
		}
		while(0);

		return $itemIDs;
	}

	function selectItemIDsSQL($keywordID, $validFolderIDs) {

		// INIT RETURN VALUE
		$itemIDs = array();

		list($column, $order) = $this->getItemSortModeSQL($keywordID);
		$query = sprintf('SELECT t1.itemID FROM `mg2db_items` t1, `mg2db_item_keyword` t2
								WHERE t2.itemID    = t1.itemID
								AND   t2.keywordID = %d
								%s
								%s
								%s
								ORDER BY t1.%s %s',
								$keywordID,
								!defined('USER_ADMIN') ? 'AND t1.folderID IN ('. implode(',', array_keys($validFolderIDs)) .')':'',
								!defined('USER_ADMIN') ? 'AND t1.position  >= 0':'',
								!defined('USER_ADMIN') ? 'AND t1.timestamp <= '. time():'',
								$column,
								$order
					);

		if ($result = mg2mysql_query($query)) {
			while ($record = mg2mysql_fetch_assoc($result)) {
				$itemIDs[] = (int)$record['itemID'];
			}
		}

		return $itemIDs;
	}

	function selectItemIDsFlat($keywordID, $validFolderIDs) {

		// INIT RETURN VALUE
		$itemIDs = array();

		do {
			// SEARCH IN ALL ITEM KEYWORD RECORDS
			settype($keywordID, 'string');
			foreach(array_keys($this->keywordIndex) as $itemID) {

				if (!array_key_exists($itemID, $this->mg2->all_items))	continue;	// $itemID doesn't exist
				if (!$keywordIDs = $this->getKeywordIDs($itemID))			continue;	// no keywords
				if (!in_array($keywordID, $keywordIDs, true))				continue;	// keywords doesn't match

				$parentID = (int)$this->mg2->all_items[$itemID][1];
				if (!array_key_exists($parentID, $validFolderIDs))			continue;	// no folder permission

				// SAVE MATCHED IDs
				$itemIDs[] = $itemID;
			}

			// SORT ITEM IDs
			$this->mg2->sortIndexDB($itemIDs, 'all_items', $this->getSortMode($keywordID));
		}
		while(0);

		return $itemIDs;
	}

	// SWITCH DATABASE (ADMIN ONLY)
	function switchDatabase($db) {
		if ($this->loadStatus === false) return -1;

		$dataOK = $indexOK = -1;

		// SWITCH TO SQL?
		if (strcasecmp($db,'sql') === 0) {
			$dataOK  = $this->writeKeywordDBSQL('data', 'all', false);
			$indexOK = $this->writeKeywordDBSQL('index', 'all', false);
		}
		// SWITCH TO FLATFILE?
		elseif (strcasecmp($db,'flat') === 0) {
			$this->optimizeKeywordData();
			$dataOK  = $this->writeKeywordFlatfile('data');
			$indexOK = $this->writeKeywordFlatfile('index');
		}

		return ($dataOK > -1 && $indexOK > -1)?
				  array($dataOK, $indexOK)
				  :
				  -1;
	}

	// BACKUP DATABASE (ADMIN ONLY)
	function backupDatabase($db) {
		if ($this->loadStatus === false)	return false;

		switch ($db) {
			case 'keywords':
				if ($this->mg2->sqldatabase) $this->optimizeKeywordData();
				$pathFlatfile = DATA_FOLDER . $this->nameDBFlatData .'_temp.php';
				$writeOK		  = $this->writeKeywordFlatfile('data', $pathFlatfile);
				break;
			case 'item_keyword':
				$pathFlatfile = DATA_FOLDER . $this->nameDBFlatIndex .'_temp.php';
				$writeOK		  = $this->writeKeywordFlatfile('index', $pathFlatfile);
				break;
			default:
				$writeOK = -1;
		}

		return ($writeOK > -1) ? $writeOK : false;
	}

	// RESTORE DATABASE (ADMIN ONLY)
	function restoreDatabase($restorePath) {

		// RESTORE PATH TO THE BACKUP FILES
		$pathDB['data']  = sprintf($restorePath, 'keywords');
		$pathDB['index'] = sprintf($restorePath, 'item_keyword');
		do {
			$result = -2;
			$this->loadStatus = is_file($pathDB['data']) ?
									  $this->loadKeywordFlatfiles($pathDB)
									  :
									  false;
			if ($this->loadStatus === false)											break;

			$result = -1;
			if (($numData  = $this->writeKeywordDB('data', 'all'))  < 0)	break;
			if (($numIndex = $this->writeKeywordDB('index', 'all')) < 0)	break;

			$resIndex = $this->deleteResidueIndex();
			$result   = array(
								'keywordData'  => $numData,
								'keywordIndex' => array('total' => $numIndex, 'residue' => $resIndex)
							);
		}
		while(0);

		return $result;
	}

	// ***************************************************************************** //
	// ****************************** PRIVATE METHODS ****************************** //
	// ***************************************************************************** //

	function getItemSortModeSQL($keywordID) {

		// GET KEYWORD SORT MODE CODE
		$sortmode = $this->getSortMode($keywordID);

		// SQL TABLE COLUMNS
		$itemTableColumns = array(
										0	=> 'itemID',
										1	=> 'folderID',
										2	=> 'title',
										3	=> 'description',
										4	=> 'timestamp',
										5	=> 'position',
										6	=> 'filename',
										7	=> 'filepath',
										8	=> 'imageWidth',
										9	=> 'imageHeight',
									  13	=> 'exifDate'
								  );

		$column = (isset($itemTableColumns[$sortmode & 15]))?
					 $itemTableColumns[$sortmode & 15]
					 :
					 'filename';
		$order  = $sortmode & 16 ? 'DESC':'ASC';

		return array($column, $order);
	}

	// CHECK	PERMISSION AND VALIDATE KEYWORD ID
	function checkKeywordID($keywordID) {

		$isKeyID = false;
		do {
			if (!isset($this->keywordData[$keywordID]))			break;

			if (!defined('USER_ADMIN')) {
				if ($this->getTimestamp($keywordID) > time())	break;
				if ($this->getPositionNumber($keywordID) < 0) 	break;
			}

			$isKeyID = true;
		}
		while(0);

		return $isKeyID;
	}

	// CLEAN AND SAVE KEYWORDS INTO AN ARRAY (ADMIN ONLY)
	function cleanKeywords($saveNewKewords=true) {

		// INIT RETURN VALUE
		$inputKeywordIDs = array();
		$newKeywordIDs   = array();
		$keysArrayLower  = array();

		// LAMBDA FUNCTION TO CONVERT KEYWORDS TO LOWER
		$keyToLower = $this->stringToLowerLambda();

		// CLEAN UP FORM VALUES
		$keysString = $this->mg2->charfix($_POST['keywords']);
		$keysString = strip_tags($keysString);
		$keysString = (string)substr($keysString, 0, 2048);
		$keysArray  = preg_split("/\s*[,\t\n]+\s*/", $keysString, -1, PREG_SPLIT_NO_EMPTY);

		// CONVERT INPUT KEYWORDS UNIQUE TO LOWER
		foreach($keysArray as $inputID=>$keyword) {
			if (!$lower = $keyToLower($keyword))    continue;	// convert to lower
			if (in_array($lower, $keysArrayLower))  continue;	// avoid duplicates

			$keysArrayLower[$inputID] = $lower;
		}

		// CHECK IF KEYWORD ALREADY EXISTS IN DATABASE
		if ($keysArrayLower) {
			foreach ($this->keywordData as $keywordID=>$record) {

				$inputID = array_search($keyToLower($record[2]), $keysArrayLower);
				if ($inputID !== false)	{
					$inputKeywordIDs[$inputID] = $keywordID;	// already existing keywords
				}
			}

			// GET INPUT IDs FOR NOT YET EXIST KEYWORDS
			$newKeywordIDs = array_diff(
										array_keys($keysArrayLower),
										array_keys($inputKeywordIDs)
								  );
		}

		// NEW KEYWORDS - WHICH NOT YET EXIST IN DATABASE
		if ($saveNewKewords) {
			$keywordID = $this->keywordData ? max(array_keys($this->keywordData)):0;
			foreach($newKeywordIDs as $inputID) {
				$newKeywordIDs[$inputID]			 = ++$keywordID;				// save new keyword id
				$this->keywordData[$keywordID][2] = $keysArray[$inputID];	// save original keyword
			}

			// IF THERE ARE NEW KEWORDS, WRITE THEM INTO DATABASE
			if (count($newKeywordIDs) > 0) {
				$writeOK = $this->writeKeywordDB('data', 'new', $newKeywordIDs);
				if ($writeOK !== -1) {
					$inputKeywordIDs += $newKeywordIDs;
				}
			}
		}

		ksort($inputKeywordIDs, SORT_NUMERIC);
		return $inputKeywordIDs;
	}

	// GET KEYWORD INDICES OF AN ITEM AS ARRAY
	function getKeywordIDs($itemID) {
		return explode(',', $this->keywordIndex[$itemID]);
	}

	// DELETE ALL KEYWORD RELATIONS OF ONE OR MORE ITEMS (ADMIN ONLY)
	function deleteItemKeywords($toDeleteIDs) {
		if (!is_array($toDeleteIDs))		return false;
		if ($this->loadStatus !== true)	return false;

		$deletedIDs	= array();
		foreach ($toDeleteIDs as $toDeleteID) {
			if (isset($this->keywordIndex[$toDeleteID])) {
				unset($this->keywordIndex[$toDeleteID]);		// delete record
				$deletedIDs[$toDeleteID] = $toDeleteID;		// store deleted id
			}
		}

		// WRITE DATABASE
		if (!empty($deletedIDs)) $this->writeKeywordDB('index', 'del', $deletedIDs);
	}

	// KEYWORD FOLDER
	function getKeywordFolder($keywordID, $col=null) {

		$folder = array(
						$keywordID,										// keyword id					[0]
						$this->getKeywordDefaults(1),				// in folder (#id)			[1]
						$this->getKeywordName($keywordID),		// keyword name				[2]
						$this->getIntroText($keywordID),			// introtext					[3]
						$this->getTimestamp($keywordID),			// timestamp (publish off)	[4]
						$this->getPositionNumber($keywordID),	// folder position			[5]
						$this->getKeywordDefaults(6),				// imageID for folder icon	[6]
						$this->getSortMode($keywordID),			// folder sort mode			[7]
						$this->getPwdHash($keywordID),			// password	hash				[8]
						$this->getfNumCols($keywordID),			// number of cols				[9]
						$this->getfNumRows($keywordID),			// number of rows				[10]
						$this->getKeywordDefaults(11)				// folder type					[11]
					);

		return (isset($folder[$col]))? $folder[$col]:$folder;
	}

	// GET INTROTEXT OF A KEYWORD
	function getIntroText($keywordID) {

		return (isset($this->keywordData[$keywordID][3]))?
				 trim($this->keywordData[$keywordID][3])
				 :
				 $this->getKeywordDefaults(3);
	}

	// GET PUBLISH TIME OF A KEYWORD
	function getTimestamp($keywordID) {

		return (isset($this->keywordData[$keywordID][4]))?
				 (int)$this->keywordData[$keywordID][4]
				 :
				 $this->getKeywordDefaults(4);
	}

	// GET POSITION OF A KEYWORD
	function getPositionNumber($keywordID) {

		return (isset($this->keywordData[$keywordID][5]))?
				 (int)$this->keywordData[$keywordID][5]
				 :
				 $this->getKeywordDefaults(5);
	}

	// GET SORT MODE OF A KEYWORD
	function getSortMode($keywordID) {

		return (isset($this->keywordData[$keywordID][7]))?
				 (int)$this->keywordData[$keywordID][7]
				 :
				 $this->getKeywordDefaults(7);
	}

	// GET PASSWORD HASH OF A KEYWORD
	function getPwdHash($keywordID) {

		return (isset($this->keywordData[$keywordID][8]))?
				 trim($this->keywordData[$keywordID][8])
				 :
				 $this->getKeywordDefaults(8);
	}

	// GET NUMBER COLS OF A KEYWORD
	function getfNumCols($keywordID) {

		return (isset($this->keywordData[$keywordID][9]))?
				 (int)$this->keywordData[$keywordID][9]
				 :
				 $this->getKeywordDefaults(9);
	}

	// GET NUMBER ROWS OF A KEYWORD
	function getfNumRows($keywordID) {

		return (isset($this->keywordData[$keywordID][10]))?
				 (int)$this->keywordData[$keywordID][10]
				 :
				 $this->getKeywordDefaults(10);
	}

	// KEYWORD DEFAULT FOLDER
	function getKeywordDefaults($col=null) {

		$folder = array(
						0	=> 0,			// keyword id
						1	=> 1,			// in folder (#id)
						2	=>	'???',	// keyword name
						3	=> '',		// introtext
						4	=> -1,		// timestamp (publish off)
						5	=> 1,			// position
						6	=> 0,			// imageID for folder icon
						7	=> 6,			// sort mode (name, ascending)
						8	=> '',		// password
						9	=> 0,			// number of cols
						10	=> 0,			// number of rows
						11	=> 9			// folder type
					);

		return (isset($folder[$col]))? $folder[$col]:$folder;
	}

	// READ DATABASE
	function loadKeywordDB() {

		// INIT KEYWORD ARRAYS
		$this->keywordIndex = array();
		$this->keywordData  = array();

		return ($this->mg2->sqldatabase)?
				 $this->loadKeywordDBSQL()
				 :
				 $this->loadKeywordFlatfiles();
	}

	// READ DATABASE - SQL
	function loadKeywordDBSQL() {

		// INIT RETURN VALUE
		$loadStatus = false;

		do {
			// GET KEYWORD - INDEX TABLE
			$query = 'SELECT * FROM `mg2db_keywords` WHERE 1';
			if (!$result = mg2mysql_query($query)) 		break;

			while ($record = mg2mysql_fetch_assoc($result)) {
				$keywordID = (int)$record['keywordID'];
				$this->keywordData[$keywordID][2]  = $record['keywordName'];
				$this->keywordData[$keywordID][3]  = $record['introText'];
				$this->keywordData[$keywordID][4]  = (int)$record['timestamp'];
				$this->keywordData[$keywordID][5]  = (int)$record['position'];
				$this->keywordData[$keywordID][7]  = (int)$record['sortMode'];
				$this->keywordData[$keywordID][8]  = $record['pwdHash'];
				$this->keywordData[$keywordID][9]  = (int)$record['itemThumbCols'];
				$this->keywordData[$keywordID][10] = (int)$record['itemThumbRows'];
			}

			// GET ITEM - KEYWORD RELATION
			$query = 'SELECT `itemID`, GROUP_CONCAT(CONVERT(`keywordID`, CHAR) ORDER BY `position` SEPARATOR ",") AS keywords
						 FROM `mg2db_item_keyword`
						 GROUP BY `itemID`
						';
			if (!$result = mg2mysql_query($query)) 		break;

			while ($record = mg2mysql_fetch_assoc($result)) {
				$itemID = (int)$record['itemID'];
				$this->keywordIndex[$itemID] = $record['keywords'];
			}

			$loadStatus = true;
		}
		while(0);

		return $loadStatus;
	}

	// READ DATABASE - FLAT FILES
	function loadKeywordFlatfiles($pathDB='default') {

		// INIT RETURN VALUE
		$loadStatus = false;

		// KEYWORD FILE PATHS
		if ($pathDB === 'default') {
			$pathDB = array (
							'data'  => $this->pathDBFlatData,
							'index' => $this->pathDBFlatIndex
						 );
		}

		do {
			if (!$this->setKeywordArray($pathDB['data'],  'keywordData'))  break;
			if (!$this->setKeywordArray($pathDB['index'], 'keywordIndex')) break;

			$loadStatus = true;
		}
		while(0);

		// LOADING DATABASES OK?
		return $loadStatus;
	}

	// LOAD AND SET KEYWORD DATA AS ARRAY
	function setKeywordArray($filePath, $dataArrayName) {

		// DOES FILE EXISTS AND NOT EMPTY
		if (is_file($filePath) && filesize($filePath)) {

			// LOCK KEYWORD FILE
			$fp     = fopen($filePath, 'rb');
			$locked = flock($fp, LOCK_SH);

			/* LOAD ITEM - KEYWORD DATA
			 * $loadedData might be:
			 *    false            - read error
			 *    empty string     - blocked by a writing operation
			 *    serialized array - OK
			 */
			$loadedData = file_get_contents($filePath);

			// UNLOCK KEYWORD FILE
			if ($locked) flock($fp, LOCK_UN);
			fclose($fp);

			// UNSERIALISE LOADED DATA AND APPLY
			$this->$dataArrayName = unserialize($loadedData);
		}

		return is_array($this->$dataArrayName) ?
				 true
				 :
				 false;
	}

	// WRITE DATABASE (ADMIN ONLY)
	function writeKeywordDB($db='data', $action='all', $itemIDs=false) {
		if(!defined('USER_ADMIN'))			return -1;
		if ($this->loadStatus !== true)	return -1;

		// SAVE SPLASH DATABASE
		return ($this->mg2->sqldatabase)?
				 $this->writeKeywordDBSQL($db, $action, $itemIDs)
				 :
				 $this->writeKeywordFlatfile($db);
	}

	// WRITE DATABASE - SQL
	function writeKeywordDBSQL($db, $action, $itemIDs) {
		// USE DIFFERENT RECORDS AND $records IS NOT AN ARRAY?
		if ($action !== 'all' && !is_array($itemIDs)) return -1;

		switch (strtolower($db)) {
			// KEYWORDS Table
			case 'data' :	$sqlTable  = 'mg2db_keywords';
								$writeData = 'keywordData';
								$getValues = 'getSQLKeywordsValues';
								$primKey   = 'keywordID';
								break;
			// ITEM ID - KEYWORD ID RELATION
			case 'index':	$sqlTable  = 'mg2db_item_keyword';
								$writeData = 'keywordIndex';
								$getValues = 'getSQLItemKeywordValues';
								$primKey   = 'itemID';
								break;
			default: return -1;
		}

		$result = -1;
		switch ($action) {
			case 'new':
			case 'upd':
				$values = array();
				foreach ($itemIDs as $id) {
					if (!array_key_exists($id, $this->$writeData)) continue;	// no data?
					$values[] = $this->$getValues($id);
				}
				$query = 'REPLACE INTO `'. $sqlTable. '` VALUES'. implode(',', $values);
				if (mg2mysql_query($query)) $result = mg2mysql_affected_rows();
				break;
			case 'all':
				$query   = 'TRUNCATE TABLE `'. $sqlTable. '`'; if (!mg2mysql_query($query)) break;

				$written = 0;
				foreach (array_keys($this->$writeData) as $id) {
					$query = 'INSERT INTO `'. $sqlTable. '` VALUES'. $this->$getValues($id);
					if (!mg2mysql_query($query)) {
						break 2;
					}

					$written += mg2mysql_affected_rows();
				}
				$result = $written;
				break;
			case 'del':
				$query = 'DELETE FROM `'. $sqlTable. '` WHERE `'. $primKey. '` IN ('. implode(',', $itemIDs) .')';
				if (mg2mysql_query($query)) $result = mg2mysql_affected_rows();
				break;
			// DELETE EXCEPT FOR ITEMS ONLY
			case 'delexcept':
				$todelete = 0;
				$deleted  = array();
				foreach ($itemIDs as $itemID) {
					do {
						$except = '';

						if (!isset($this->{$writeData}[$itemID]))		break;
						if (!$data = $this->{$writeData}[$itemID])	break;

						$except = 'AND `keywordID` NOT IN ('. $data .')';
					}
					while(0);
					$todelete++;
					$query = sprintf('DELETE FROM `%s` WHERE `%s`=%d %s',
									$sqlTable,
									$primKey,
									$itemID,
									$except
								);
					if (mg2mysql_query($query)) $deleted[] = mg2mysql_affected_rows();
				}
				if (count($deleted) === $todelete) $result = array_sum($deleted);
				break;
			default: $this->mg2->status = 'Error in '. $sqlTable. ' sql database';
		}
		return $result;
	}

	// GET VALUES TO WRITE INTO SQL 'mg2db_keywords' TABLE
	function getSQLKeywordsValues($keywordID) {
		return sprintf('(%d, "%s", "%s", %d, %d, %d, "%s", %d, %d)',
					 $keywordID,																	// keyword id
					 mg2mysql_real_escape_string($this->getKeywordName($keywordID)),	// keyword name
					 mg2mysql_real_escape_string($this->getIntroText($keywordID)),		// intro text
					 $this->getTimestamp($keywordID),										// timestamp
					 $this->getPositionNumber($keywordID),									// position
					 $this->getSortMode($keywordID),											// sort mode;
					 mg2mysql_real_escape_string($this->getPwdHash($keywordID)),		// password hash;
					 $this->getfNumCols($keywordID),											// rhumb cols
					 $this->getfNumRows($keywordID)											// thumb rows
				 );
	}

	// GET VALUES TO WRITE INTO SQL 'mg2db_item_keyword' TABLE
	function getSQLItemKeywordValues($itemID) {

		$values = array();
		foreach($this->getKeywordIDs($itemID) as $position=>$keywordID) {
			$values[] = sprintf('(%d, %d, %d)',
								$itemID,
								$keywordID,
								$position
							);
		}

		return implode(',', $values);		
	}

	// WRITE DATA - FLAT FILE
	function writeKeywordFlatfile($db, $pathDB='default') {

		// INIT WRITE STATUS
		$writeOK = false;

		switch (strtolower($db)) {
			// KEYWORD ID - KEYWORD ARRAY
			case 'data' :	$numWriteData = $this->getNumKeywords();
								$pathFlatfile = ($pathDB === 'default')? $this->pathDBFlatData:$pathDB;
								$nameFlatfile = $this->nameDBFlatData;
								$writeData	  = 'keywordData';
								break;
			// ITEM ID - KEYWORD ID ARRAY
			case 'index':	$numWriteData = $this->getNumItemKeywords();
								$pathFlatfile = ($pathDB === 'default')? $this->pathDBFlatIndex:$pathDB;
								$nameFlatfile = $this->nameDBFlatIndex;
								$writeData	  = 'keywordIndex';
								break;
			default: return -1;
		}

		// NO KEYWORD RECORDS
		if ($numWriteData < 1)
		{
			$writeOK = is_file($pathFlatfile) ?
						  unlink($pathFlatfile)			// there's nothing to write, delete keyword file
						  :
						  true;								// there's nothing to write, but no file
		}
		// WRITE KEYWORD DATA FILE
		elseif (($fp = @fopen($pathFlatfile, 'wb'))		||
				  (rename($pathFlatfile, sprintf('%s%d_%s.php',
													 BACKUP_FOLDER,
													 time(),
													 $nameFlatfile
												 ))					&&
					($fp = fopen($pathFlatfile, 'wb')))
				 )
		{
			flock($fp, LOCK_EX);		// do an exclusive lock
			$writeOK = fwrite($fp, serialize($this->$writeData));
			flock($fp, LOCK_UN);		// release the lock
			fclose($fp);
		}

		return $writeOK ? $numWriteData:-1;
	}

	// OPTIMZE KEYWORD DATA FOR FLAT FILE SAVING
	function optimizeKeywordData($items=false) {

		if (!$items) $items = array_keys($this->keywordData);

		$defaultKeyFolder = $this->getKeywordDefaults();
		foreach($items as $item) {
			$this->keywordData[$item] = array_diff_assoc(
													$this->keywordData[$item],
													$defaultKeyFolder
												 );
		}
	}

	// BUILD LAMBDA FUNCTION TO CONVERT KEYWORDS TO LOWER
	function stringToLowerLambda() {
		$convertCode = 'return ';
		$convertCode.= $this->mg2->getCharsetSupport()?
							'mb_convert_case($keyword,MB_CASE_LOWER,\''. $this->mg2->charset .'\');'
							:
							'strtolower($keyword);';
		return create_function('$keyword', $convertCode);
	}
}
?>