<?php
/**
 * CONVERTER CLASS, IMPLEMENTATION SHOULD BE AS SINGLETON
 *
 * CONVERTS MG2 0.5.0/0.5.1 AND KH_MOD 0.1.0 DATABASES TO KH_MOD 0.4.2
 * kh_mod 0.4.2, add
 */
class MG2AdminConverter {

	var $mg2;	// parent object reference

	/**
	 * INITIALIZE CONVERTER
	 */
	function MG2AdminConverter(&$parent) {
		$this->mg2 = &$parent;
	}

	/**
	 * ITEM DATABASE
	 * FOR MG2 0.5.1 AND kh_mod < 0.2.0 b2
	 */
	function updateItemData() {

		// GET START TIME
		$start = $this->microtime_float();

		$writeOK	= false;
		$iDB_old	= 'mg2db_idatabase.php';

		if (count($this->mg2->all_items) > 0) {
			$this->mg2->displayStatus(($this->mg2->sqldatabase)?
				'Item data in sql table \'mg2db_items\' already exists!'
				:
				'Item data in \'data/'. $iDB_old .'\' already exists!', 2
			);
			return false;
		}

		do {
			$this->mg2->all_items = array();
			if (!is_file($iDB_old)) {
				$message  = 'Didn\'t find any item file to import from the gallery root!';
				$errLevel = 1;
				break;
			}
			if (!$fp = @fopen($iDB_old,'rb'))	break;	// cannot open data file
			if (!is_resource($fp))					break;	// $fp is no resource

			$this->mg2->itemAutoID = (int)fgets($fp, 16);
			while (!feof($fp)) {
				if (fgets($fp, 2) !== '*')					continue; // no data row?
				$record = fgetcsv($fp, 4600, '*');
				if (($itemID = (int)$record[0]) < 1)	continue; // invalid item id
				$this->mg2->all_items[$itemID] = $record;
			}
			fclose($fp);

			foreach ($this->mg2->all_items as $key=>$record) {
				$this->mg2->all_items[$key][0]  = $key;
				$this->mg2->all_items[$key][1]  = max((int)$record[2], 1);
				$this->mg2->all_items[$key][2]  = str_replace("\t",'   ',trim($record[3]));
				$this->mg2->all_items[$key][3]  = str_replace("\t",'   ',trim($record[4]));
				$this->mg2->all_items[$key][4]  = (int)$record[10];
				$this->mg2->all_items[$key][5]  = (isset($record[11]))? (int)$record[11]:1;
				$this->mg2->all_items[$key][6]  = trim($record[1]);
				$this->mg2->all_items[$key][7]  = (isset($record[12]))? trim($record[12]):'';
				$this->mg2->all_items[$key][8]  = (int)$record[6];
				$this->mg2->all_items[$key][9]  = (int)$record[7];
				$this->mg2->all_items[$key][10] = (int)$record[8];
				$this->mg2->all_items[$key][11] = (int)$record[9];
				$this->mg2->all_items[$key][12] = (int)$record[5];
				$this->mg2->all_items[$key][13] = 0;
				$this->mg2->all_items[$key][14] = '';
				$this->mg2->all_items[$key][15] = 0;
				$this->mg2->all_items[$key][16] = $this->mg2->getContentCode($this->mg2->getFileExtension($key));
			}

			// CHECK AUTO ID AND WRITE ITEM DATABASE
			if ($this->mg2->all_items) {
				define('ItemDataOK', true);				

				$maxID = max(array_keys($this->mg2->all_items));
				if ($maxID > $this->mg2->itemAutoID) {
					$this->mg2->itemAutoID = $maxID;
				}
				$writeOK = $this->mg2->write_iDB('all');
			}

			// BUILD STATUS MESSAGE
			if ($writeOK !== -1) {
				$message  = sprintf('Item database imported %d records, it took %.3f sek.',
									count($this->mg2->all_items),
									$this->microtime_float() - $start
								);
				$errLevel = 0;
			}
			else {
				$message  = 'Couldn\'t import any item record!';
				$errLevel = 3;
			}
		}
		while(0);

		// DISPLAY STATUS MESSAGE
		$this->mg2->displayStatus($message, $errLevel);
	}

	/**
	 * FOLDER DATABASE
	 * FOR MG2 0.5.1 AND kh_mod < 0.2.0 b2
	 */
	function updateFolderData() {

		// GET START TIME
		$start = $this->microtime_float();

		$writeOK	= false;
		$fDB_old	= 'mg2db_fdatabase.php';

		if (count($this->mg2->all_folders) > 1) {
			$this->mg2->displayStatus(($this->mg2->sqldatabase)?
				'Folder data in sql table \'mg2db_folders\' already exists!'
				:
				'Folder data in \'data/'. $fDB_old .'\' already exists!', 2
			);
			return false;
		}

		do {
			$this->mg2->all_folders = array();
			if (!is_file($fDB_old)) {
				$message  = 'Didn\'t find any folder file to import from the gallery root!';
				$errLevel = 1;
				break;
			}
			if (!$fp = @fopen($fDB_old,'rb'))	break;	// cannot open data file
			if (!is_resource($fp))					break;	// $fp is no resource

			$this->mg2->folderAutoID = (int)fgets($fp,16);
			while (!feof($fp)) {
				if (fgets($fp,2) !== '*')					continue;	// no data row?
				$record = fgetcsv($fp, 4600, '*');
				if (($folderID = (int)$record[0]) < 1)	continue;	// invalid folder id
				$this->mg2->all_folders[$folderID] = $record;
			}
			fclose($fp);

			foreach ($this->mg2->all_folders as $key=>$record) {
				$this->mg2->all_folders[$key][0] = $key;

				// ROOT FOLDER
				if ($key === 1) {
					$this->mg2->all_folders[$key][1] = 'root';
					$this->mg2->all_folders[$key][2] = (trim($record[2])==='root')?
																	''
																	:
																	str_replace("\t",'   ',trim($record[2]));
				}
				// ANY OTHER FOLDER
				else {
					$this->mg2->all_folders[$key][1] = max((int)$record[1], 1);
					$this->mg2->all_folders[$key][2] = str_replace("\t",'   ',trim($record[2]));
				}
				$this->mg2->all_folders[$key][3]	 = str_replace("\t",'   ',trim($record[10]));
				$this->mg2->all_folders[$key][4]  = (int)$record[6];
				$this->mg2->all_folders[$key][5]  = (isset($record[11]))? (int)$record[11]:1;
				$this->mg2->all_folders[$key][6]  = (isset($record[7]))?  $this->_getimage($record[7]):-1;
				$this->mg2->all_folders[$key][7]	 = $this->_getsortby($record[3]);		// sort by
				$this->mg2->all_folders[$key][7]	|= (int)$record[4] << 4;					// sort mode
				$this->mg2->all_folders[$key][8]  = trim($record[5]);
				$this->mg2->all_folders[$key][9]  = 0;
				$this->mg2->all_folders[$key][10] = 0;
				$this->mg2->all_folders[$key][11] = 0;
				$this->mg2->all_folders[$key][12] = '';
			}

			// CHECK AUTO ID AND WRITE FOLDER DATABASE
			if ($this->mg2->all_folders) {
				define('FolderDataOK', true);

				$maxID = max(array_keys($this->mg2->all_folders));
				if ($maxID > $this->mg2->folderAutoID) { 
					$this->mg2->folderAutoID = $maxID;
				}
				$writeOK = $this->mg2->write_fDB('all');
			}

			// BUILD STATUS MESSAGE
			if ($writeOK !== -1) {
				$message  = sprintf('Folder database imported %d records, it took %.3f sek.',
									count($this->mg2->all_folders),
									$this->microtime_float() - $start
								);
				$errLevel = 0;
			}
			else {
				$message  = 'Couldn\'t import any folder record!';
				$errLevel = 3;
			}
		}
		while(0);

		// DISPLAY STATUS MESSAGE
		$this->mg2->displayStatus($message, $errLevel);
	}

	/**
	 * COMMENT FILES
	 * FOR MG2 0.5.1 AND kh_mod < 0.2.0 b2
	 */
	function updateCommentData() {

		// INIT START VALUES
		$read		  = 0;
		$converted = 0;
		$saved	  = 0;
		$start	  = $this->microtime_float();

		if ($files = glob($this->mg2->imagefolder .'/*.comment')) {

			require_once(INC_FOLDER .'mg2_comments.php');
			$Comments = &$this->mg2->getInstance('MG2Comments');
			foreach ($files as $commfile) {
				$read++;
				if (!$oldComments = $this->loadOldComments($commfile))	continue;

				// CONVERT RECORDS
				$commentID	  = 0;
				$convComments = array();
				foreach ($oldComments as $record) {
					$commentID++;

					// NEW COMNMENT
					if (isset($record['new'])) {
						$record['new'][0] = $commentID;
						$convComments[$commentID] = $record['new'];
					}
					// OLD COMMENT
					else {
						$convComments[$commentID] = array(
																$commentID,
																str_replace("\t",'   ',trim($record['old'][1])),
																trim($record['old'][2]),
																str_replace("\t",'   ',trim($record['old'][3])),
																$record['old'][0],
																-1,
																0,
																1
															 );
					}
				}
				$converted++;

				// WRITE CONVERTED COMMENT DATA
				if ($convComments && $Comments->importComments($convComments, $commfile)) $saved++;
			}
		}
		$takes = round($this->microtime_float() - $start, 3);

		// GET AND DISPLAY MESSAGES
		if ($read === 0) {
			$message = 'Found no comment files to convert in \''.$this->mg2->imagefolder.'\' directory!';
			$errtype = 1; // notice
		}
		elseif ($converted == 0) {
			$message = 'Read '. $read .' files, but no old comments to convert!';
			$errtype = 1; // notice
		}
		else {
			$message = 'Comment database converted '. $converted .' of '. $read;
			$message.= ' files, ';
			if ($converted > $saved) {
				$message.= ($converted - $saved) .' write error(s)!';
				$errtype = 3; // error
			} else {
				$message.= 'it took '. $takes .' sek.';
				$errtype = 0; // ok
			}
		}
		$this->mg2->displayStatus($message, $errtype);
	}

	/**
	 * CLICK-COUNTER PLUG-IN DATABASE
	 * FOR MG2 0.5.0/0.5.1 AND kh_mod < 0.3.0 b5
	 */
	function updateClickData() {

		// GET START TIME
		$start = $this->microtime_float();

		// LOAD AND INITIALIZE ADMIN COUNTER CLASS
		include_once(INC_FOLDER .'mg2_counter.php');
		include_once(INC_FOLDER .'mg2admin_counter.php');
		$Counter = &$this->mg2->getInstance('MG2AdminCounter');

		// COUNTER DATABASE ALREADY EXISTS?
		if ($Counter->getNumCounters()) {
			$this->mg2->displayStatus(empty($this->mg2->sqldatabase)?
						'Counter data in \'data/mg2db_counter.php\' already exists!'
						:
						'Counter data in sql table \'mg2db_counter\' already exists!', 2
					);
			return false;
		}

		// CONVERT COUNTER DATA
		$plugin_data = $this->loadOldCounterData();

		do {
			// COULD NOT READ PLUGIN DATABASE FILE
			if ($plugin_data === -1) {
				$message = 'Couldn\'t read click counter plug-in file \'database.txt\'!';
				$errtype = 3;	// error
				break;
			}

			if ($plugin_data === -2) {
				$message = 'Couldn\'t import counter plug-in, because file \'database.txt\' doesn\'t exist!';
				$errtype = 1;	// notice
				break;
			}

			// CONVERT PLUGIN DATA
			foreach ($plugin_data as $key=>$item) {
				if ($key < 1) { unset($plugin_data[$key]); continue; }	// no valid image index
				$plugin_data[$key] = array($key, $item, 0, 0);				// new counter record
			}

			// NO VALID DATA COUNTER DATA
			if (count($plugin_data) < 1) {
				$message = 'There are no valid counter plug-in data to import!';
				$errtype = 1; // notice
				break;
			}

			// IMPORT PLUGIN DATA
			if ($num = $Counter->importDatabase($plugin_data)) {
				$message = $num.' of '. count($plugin_data) .' click counter '.
							  'plug-in entries imported, it took '.
							  round($this->microtime_float() - $start, 3) .' sek.';
				$errtype = 0;	// ok
				break;
			}

			// UNDEFINED ERROR
			$message = 'Couldn\'t import click counter plug-in database!';
			$errtype = 3;	// error
		}
		while(0);

		$this->mg2->displayStatus($message, $errtype);
	}

	/**
	 * CHANGE STRUCTURE TO SET LAST CLICK TIMESTAMP
	 * FOR kh_mod 0.4.0 rc1 TO 0.4.1
	 */
	function updateChronicleData(&$chronicleData) {

		// LOAD AND INITIALIZE ADMIN COUNTER CLASS
		include_once(INC_FOLDER .'mg2_counter.php');
		include_once(INC_FOLDER .'mg2admin_counter.php');
		$Counter = &$this->mg2->getInstance('MG2AdminCounter');

		$Counter->loadChronicleData();
		$Counter->updateChronicleStruct();
		$Counter->writeChronicle();
	}

	/**
	 * CLEAN UP SETTINGS FOR UPDATE AND RESTORE
	 * FOR ALL kh_mod VERSIONS
	 */
	function cleanSettingData($Settings) {

		// CLEAR NUMERICS
		settype($Settings->installdate, 'integer');
		settype($Settings->statisticsets, 'integer');
		settype($Settings->navtype, 'integer');
		settype($Settings->layoutsetting, 'integer');
		settype($Settings->extendedset, 'integer');

		// CLEAR 'extensions', NORMALLY FOR kh_mod BEFORE 0.4.1
		if (is_string($Settings->extensions)) {
			$regexp  = '/[\s,]+/';
			$oldExts = strtolower($Settings->extensions);
			$setExts	= array_intersect(
								$this->mg2->getSupportedExtensions(),
								preg_split($regexp, $oldExts, -1, PREG_SPLIT_NO_EMPTY)
						  );
			$Settings->extensions = array_sum(array_keys($setExts));
		}

		// CLEAR UNITS FOR 'canvasWidth', 'contentWidth' AND 'introWidth'
		$Settings->canvasWidth  = sprintf('%d%s',
												$Settings->canvasWidth,
												(substr($Settings->canvasWidth, -1) === '%')? '%':'px'
										  );
		$Settings->contentWidth = sprintf('%d%s',
												$Settings->contentWidth,
												(substr($Settings->contentWidth, -1) === '%')? '%':'px'
										  );
		$Settings->introWidth	= sprintf('%d%s',
												$Settings->introWidth,
												(substr($Settings->introWidth, -1) === '%')? '%':'px'
										  );

		// CLEAR HTML CONTENT
		$Settings->gallerytitle = $this->mg2->cleanHTMLContent(
																$Settings->gallerytitle,
																array('b','i','a','br','img','span','div')
														  );
		$Settings->websitetext	= $this->mg2->cleanHTMLContent(
																$Settings->websitetext,
																array('b','i','img','span','div')
														  );
		$Settings->copyright    = $this->mg2->cleanHTMLContent(
																$Settings->copyright,
																array('b','i','a','br','img','span','div')
														  );

		return $Settings;
	}

	/**
	 * RENAME FOLDER AND ITEM FLAT FILE
	 * FOR kh_mod 0.2.0 b2 up to 0.4.0 b4
	 */
	function renameOldDatabase() {

		// INIT START VALUES
		$folderSumOK  = $itemSumOK  = 0;
		$folderErrors = $itemErrors = array();

		// GET BACKUP FILES
		$regexp = '/^mg2db_(i|f)database(_temp)?\.php$/i';

		if ($workdir = opendir(DATA_FOLDER))
		while (false !== ($file = readdir($workdir))) {
			if ($file{0} === '.')							continue;
			if (!preg_match($regexp, $file, $items))	continue;

			switch ($items[1]) {
				// RENAME FOLDER FLAT FILE
				case 'f': if (!rename(DATA_FOLDER . $file, DATA_FOLDER . 'mg2db_folders'. $items[2] .'.php')) {
								$folderErrors[] = $file;
							 }
							 $folderSumOK++;
							 break;
				// RENAME ITEM FLAT FILE
				case 'i': if (!rename(DATA_FOLDER . $file, DATA_FOLDER . 'mg2db_items'. $items[2] .'.php')) {
								$itemErrors[] = $file;
							 }
							 $itemSumOK++;
			}
		}

		// LOG ERROR MESSAGE
		$messageError = 'Couldn\'t rename %d of %d folder flat files to \'%s\'!';
		if ($nErrors = count($folderErrors)) {
			$this->mg2->log(sprintf($messageError, $nErrors, $folderSumOK, 'mg2db_folders'));
		}
		if ($nErrors = count($itemErrors)) {
			$this->mg2->log(sprintf($messageError, $nErrors, $itemSumOK, 'mg2db_items'));
		}

		return (!($folderErrors || $itemErrors))?
				 true
				 :
				 array('folders'=>$folderErrors, 'items'=>$itemErrors);
	}

	/**
	 * RENAME FOLDER AND ITEM FLAT FILE
	 * FOR kh_mod 0.2.0 b2 up to 0.4.0 b4
	 */
	function renameOldBackups() {

		// GET BACKUP FILES
		$renOK  = 0;
		$errors = array();
		$regexp = '/^([0-9]{10})_mg2db_(\w{7,12})\.php$/';

		if ($workdir = opendir(DATA_FOLDER))
		while (false !== ($file = readdir($workdir))) {
			if ($file{0} === '.')							continue;
			if (!preg_match($regexp, $file, $items))	continue;

			switch ($items[2]) {
				case 'splashes':
				case 'keywords':
				case 'item_keyword':
				case 'settings':
				case 'counter':
				case 'fdatabase':
				case 'idatabase':
					$this->renameOldBackupAction($items[1], $file, $items[2]) ?
					$renOK++
					:
					$errors[]=$file;
			}
		}

		// LOG ERROR MESSAGE
		if ($numErr = count($errors)) {
			$this->mg2->log(sprintf('Couldn\'t rename %d of %d backup files!', $numErr, $numErr + $renOK));
		}

		return (!$errors)?
				 true
				 :
				 array('backups'=>$errors);
	}

	/**
	 * RENAME FOLDER AND ITEM FLAT FILE
	 * FOR kh_mod 0.2.0 b2 up to 0.4.0 b4
	 */
	function renameOldBackupAction($time, $file, $name) {

		return rename(
					DATA_FOLDER . $file,
					BACKUP_FOLDER . $time .'_mg2db_'. $name .'.php'
				 );
	}

	/**
	 * SET ALL CONTENT TYPES IN ITEM DATABASE
	 * FOR kh_mod 0.2.0 b2 up to 0.4.0
	 */
	function setContentType() {

		// GET START TIME
		$start = $this->microtime_float();

		// READ DATABASE
		list($readFolders, $readItems) = $this->mg2->readDB();

		// ARE THERE ITEM RECORDS?
		if ($readItems > 0) {
			define('ItemDataOK', true);

			// CHECK BOOKMARKED AND SET CONTENT TYPE
			foreach (array_keys($this->mg2->all_items) as $key) {
				$contentCode = $this->mg2->getContentCode($this->mg2->getFileExtension($key));

				// ITEM BOOKMARKED
				if (!isset($this->mg2->all_items[$key][15])
					|| ((int)$this->mg2->all_items[$key][15] < 1)
					|| ((int)$this->mg2->all_items[$key][15] === $contentCode))
				{
					$this->mg2->all_items[$key][15] = 0;	// currently "bookmarked"
				}
				// CONTENT TYPE AND IMPORT STATUS
				$this->mg2->all_items[$key][16] = $contentCode;
			}

			// WRITE ITEM DATABASE
			if ($this->mg2->write_iDB('all') !== -1) {
				$message  = sprintf('Content type set in each item data record, it took %.3f sek.',
									$this->microtime_float() - $start
								);
				$errLevel = 0;
			}
			else {
				$message  = 'Couldn\'t set content type in any item record!';
				$errLevel = 3;
			}

			// DISPLAY STATUS MESSAGE
			$this->mg2->displayStatus($message, $errLevel);
		}
	}

	/**
	 * CONVERT "SORT BY" (FOLDER DATA RECORD)
	 * FOR MG2 0.5.1 AND kh_mod < 0.2.0 b2
	 */
	function _getsortby($idx) {
		switch ((int)$idx) {
			case  1:	$idx =  6; break;
			case  3:	$idx =  2; break;
			case  4:	$idx =  3; break;
			case  5:	$idx = 12; break;
			case  6:	$idx =  8; break;
			case  7:	$idx =  9; break;
			case 10:	$idx =  4; break;
			case 11:	$idx =  5; break;
			default: $idx =  6;
		}
		return $idx;
	}

	/**
	 * CONVERT THUMB NAME TO IMAGE ID
	 */
	function _getimage($thumb) {
		$extension = strrchr(trim($thumb), '.');
		$thumbName = basename($thumb, $extension);
		$imageIDs  = array();
		if (substr($thumbName, -6) === '_thumb') {
			$fileName = substr($thumbName, 0, -6) . $extension;
			$imageIDs = $this->mg2->selectIndexDB($fileName, 'all_items', 6);
		}
		return current($imageIDs) ? current($imageIDs):-1;	// -1 means random image as icon
	}

	/**
	 * GET CLICK-COUNTER PLUG-IN DATABASE
	 */
	function loadOldCounterData() {

		// INIT VALUES
		$data = array();
		$file = 'database.txt';

		// VERIFY PLUG-IN COUNTER FILE
		if (!is_readable($file)) return (is_file($file))? -1:-2;

		// READ PLUG-IN COUNTER FILE
		$fp = @fopen('database.txt', 'rb');
		while ($fp && !feof($fp)) {
			$line = fgets($fp, 128);
			if (empty($line))								continue;	// no data row?
			$record = explode('|', $line);
			if (($itemID = (int)$record[0]) < 1)	continue; // invalid item id
			$data[$itemID] = (int)$record[1];
		}
		fclose($fp);
		return $data;
	}

	/**
	 * GET OLD COMMENT ENTRIES
	 */
	function loadOldComments($commfile) {
		$comments = array();
		$old = $new = 0;
		if (is_readable($commfile)) {
			$fp = @fopen($commfile, 'rb');
			while ($fp && !feof($fp)) {
				$linemark = fgets($fp, 2);
				if ($linemark === '#') {
					$record = fgetcsv($fp, 4600, "\t");
					if ((int)$record[0] < 1)	continue;	// invalid comment id
					if (count($record)  < 5)	continue;	// invalid new comment entry

					$comments[] = array('new'=>$record); $new++;
				}
				elseif ($linemark === '*') {
					$record = fgetcsv($fp, 4600, "*");
					if ((int)$record[0] < 1)	continue;	// invalid comment date
					if (count($record)  < 4) 	continue;	// invalid old comment entry

					$comments[] = array('old'=>$record); $old++;
				}
			}
			fclose($fp);
		}
		elseif (is_file($commfile)) {
			return false;
		}
		return ($old > 0)? $comments:false;
	}

	/**
	 * MOVE DIR '_global_' TO 'addons'
	 * FOR kh_mod >= 0.3.0 b6 UP TO 0.4.0 b4
	 */
	function moveAddonDir() {

		// INIT START VALUES
		$oldAddonPath = './skins/_global_/';
		$whitelist	  = 'checklists/whitelist.php';
		$blacklist	  = 'checklists/blacklist.php';
		$captchafile  = 'captcha/mg2_captcha.php';
		$checklistOK  = $captchaOK = true;

		// CHECK ADDON CHECKLISTS
		foreach (array($whitelist, $blacklist) as $list) {
			if (is_file($oldAddonPath . $list)) {
				if (!is_file(ADDON_FOLDER . $list)	||
					 sha1_file(ADDON_FOLDER . $list) !== sha1_file($oldAddonPath . $list))
				{
					if (is_file(ADDON_FOLDER . $list)) unlink(ADDON_FOLDER . $list);
					$checklistOK = rename($oldAddonPath . $list, ADDON_FOLDER . $list);
				}
			}
		}

		// CHECK ADDON CAPTCHA
		if (is_file($oldAddonPath . $captchafile)	&&
			 !is_dir(ADDON_FOLDER .'captcha'))
		{
			$captchaOK = rename($oldAddonPath .'captcha', ADDON_FOLDER .'captcha');
		}

		return array($checklistOK, $captchaOK);
	}

	/**
	 * GET MILLISEK AS FLOAT
	 */
	function microtime_float() {
		list($usec, $sec) = explode(' ', microtime());
		return ((float)$usec + (float)$sec);
	}
}
?>